require("util")
require("lib/utilities")
require("lib/strings")
require("lib/conditions")

local EOL = "\r\n"
local CSV_SEP = ";"

defines.wire_type.combined = 10

local function initGlob()
	log("Loaded Globals")
	global = global or {}
	global.cls = global.cls or {}
	global.cl_status = global.cl_status or {
		logging = false,
		to_dump = {},
		ignored_signals = {},
		trigger_off_after = false,
		trigger_off_ticks = ""
	}
end

-- start logging if it is stopped
local function trigger_on(event, trigger_name)
	if global.cl_status.logging == false then
		global.cl_status.logging = true
		global.cl_status.last_trigger_on = event.tick
		global.cl_status.current_log = {}
		global.cl_status.current_log.start_tick = event.tick
		global.cl_status.current_log.trigger_on_name = trigger_name
		global.cl_status.current_log.data = {}
		global.cl_status.current_log.changes = {} --key is tick of some change in signals, value is always true
		global.cl_status.current_log.count = 0
		global.cl_status.current_log.last_logged_tick = 0
		 for _, circuit_logger in ipairs(global.cls) do
			circuit_logger.last_signals = {
				[defines.wire_type.red] = {},
				[defines.wire_type.green] = {}
			}

			circuit_logger.current_log = {
				data = {},
				start_tick = event.tick,
				used_signals = { [defines.wire_type.red] = {}, [defines.wire_type.green] = {} }
			}
		end

		update_gui_players()
	end
end

-- stop logging if it was started and marks logged signals to be dumped into a file
local function trigger_off(event, trigger_name)
	if global.cl_status.logging == true then
		global.cl_status.logging = false
		global.cl_status.current_log.end_tick = event.tick
		global.cl_status.current_log.trigger_off_name = trigger_name
		table.insert(global.cl_status.to_dump, global.cl_status.current_log)
		global.cl_status.current_log = nil
		global.cl_status.last_log = {trigger_off_name = trigger_name}
		for _, circuit_logger in ipairs(global.cls) do
			circuit_logger.last_signals = { [defines.wire_type.red] = {}, [defines.wire_type.green] = {} }
			circuit_logger.to_dump = {} or circuit_logger.to_dump
			table.insert(circuit_logger.to_dump, circuit_logger.current_log)
			circuit_logger.current_log = nil
		end

		update_gui_players()
	end
end

local function get_circuit_logger_log(circuit_logger, start_tick)
	local log = circuit_logger.to_dump[1]
	while log ~= nil do
		if log.start_tick==start_tick then
			return log
		elseif log.start_tick>start_tick then
			return nil
		end

		table.remove(circuit_logger.to_dump,1)
		log = circuit_logger.to_dump[1]
	end
	return nil
end

local function add_csv_text(text, csv_line, csv_columns, position)
	while csv_columns<position do
		csv_line = csv_line .. CSV_SEP
		csv_columns = csv_columns+1
	end
	csv_line = csv_line .. text .. CSV_SEP
	csv_columns = csv_columns+1
	return csv_line, csv_columns
end

local function get_signals_header(signals, csv_line, csv_columns, csv_position, circuit_logger)
	local first = false
	local aliases = circuit_logger.signal_aliases or {}
	local binary = circuit_logger.binary_signals or {}
	for name, _ in pairs(signals) do
		if (binary[name] ~= nil) then
			for i, binName in pairs(binary[name]) do
				if (not first) then
					first = true
					csv_line, csv_columns = add_csv_text(
						binName .. "(" .. i .. ")",
						csv_line,
						csv_columns,
						csv_position
					)
				else
					csv_line = csv_line .. binName .. "(" .. i .. ")" .. CSV_SEP
					csv_columns = csv_columns + 1
				end
			end
		elseif (aliases[name] ~= nil) then
			name = aliases[name]
		end

		if (not first) then
			first = true
			csv_line, csv_columns = add_csv_text(name, csv_line, csv_columns, csv_position)
		else
			csv_line = csv_line .. name .. CSV_SEP
			csv_columns = csv_columns + 1
		end
	end

	return csv_line, csv_columns
end

local function combine_signals(signalsA, signalsB)
	combined = {}
	for name, count in pairs(signalsA) do
		combined[name] = count
	end

	for name, count in pairs(signalsB) do
		if (combined[name] ~= nil) then
			combined[name] = combined[name] + count
		else
			combined[name] = count
		end
	end

	return combined
end

local function get_dump_headers(start_tick)
	local headerLoggers = CSV_SEP .. "Logger name:" .. CSV_SEP
	local headerWires = CSV_SEP .. "Wire:" .. CSV_SEP
	local headerSignals = "Game tick" .. CSV_SEP .. "Play Time" .. CSV_SEP
	local headerLoggersC = 2
	local headerWiresC = 2
	local headerSignalsC = 2
	for i, circuit_logger in ipairs(global.cls) do
		log = get_circuit_logger_log(circuit_logger, start_tick)
		if log ~= nil and (circuit_logger.log_red or circuit_logger.log_green or circuit_logger.log_combined) then
			circuit_logger.current_dump_log = log
			local name = circuit_logger.name
			if circuit_logger.name == nil or circuit_logger.name == "" then name = i end
			headerLoggers, headerLoggersC = add_csv_text(
				name,
				headerLoggers, headerLoggersC,
				math.max(headerWiresC, headerSignalsC)
			)

			if (circuit_logger.log_red) then
				local signals = log.used_signals[defines.wire_type.red]
				headerWires, headerWiresC = add_csv_text(
					"Red",
					headerWires, headerWiresC,
					math.max(headerLoggersC-1, headerSignalsC)
				)

				headerSignals, headerSignalsC = get_signals_header(
					signals,
					headerSignals, headerSignalsC, headerWiresC-1,
					circuit_logger
				)
			end
			if (circuit_logger.log_green) then
				local signals = log.used_signals[defines.wire_type.green]
				headerWires, headerWiresC = add_csv_text(
					"Green",
					headerWires, headerWiresC,
					math.max(headerLoggersC-1, headerSignalsC)
				)

				headerSignals, headerSignalsC = get_signals_header(
					signals,
					headerSignals, headerSignalsC, headerWiresC-1,
					circuit_logger
				)
			end
			if (circuit_logger.log_combined) then
				local signals = combine_signals(
					log.used_signals[defines.wire_type.red],
					log.used_signals[defines.wire_type.green]
				)

				headerWires, headerWiresC = add_csv_text(
					"Combined",
					headerWires, headerWiresC,
					math.max(headerLoggersC-1, headerSignalsC)
				)

				headerSignals, headerSignalsC = get_signals_header(
					signals,
					headerSignals, headerSignalsC, headerWiresC-1,
					circuit_logger
				)

				log.used_signals[defines.wire_type.combined] = signals
			end
		end
	end

	return headerLoggers .. EOL .. headerWires .. EOL .. headerSignals
end

local function update_logged_signals_values(signals, changeData)
	if (changeData ~= nil) then
		for name, count in pairs(changeData) do
			signals[name] = count
		end
	end
end

local function get_signal_values_csv(signals, binary_signals_def)
	local csv = ""
	for name, count in pairs(signals) do
		if (binary_signals_def ~= nil and binary_signals_def[name] ~= nil) then
			local bin = to_binary(count)
			for n, _ in pairs(binary_signals_def[name]) do
				i = tostring(n + 1)
				v = 0
				if (bin[i] ~= nil) then
					v = bin[i]
				end
				csv = csv .. v .. CSV_SEP
			end
		end
		csv = csv .. count .. CSV_SEP
	end
	if csv == "" then csv = CSV_SEP end
	return csv
end

local function get_dump_line(change_tick, start_tick)
	local line = change_tick .. CSV_SEP .. get_fmt_play_time(change_tick) .. CSV_SEP
	for i, circuit_logger in ipairs(global.cls) do
		local log = circuit_logger.current_dump_log
		if log ~= nil and (circuit_logger.log_red or circuit_logger.log_green or circuit_logger.log_combined) then
			local data_change = log.data[change_tick] or {}
			local used_signals = log.used_signals
			if (circuit_logger.log_red) then
				update_logged_signals_values(
					used_signals[defines.wire_type.red],
					data_change[defines.wire_type.red]
				)

				line = line .. get_signal_values_csv(
					used_signals[defines.wire_type.red],
					circuit_logger.binary_signals
				)
			end
			if (circuit_logger.log_green) then
				update_logged_signals_values(
					used_signals[defines.wire_type.green],
					data_change[defines.wire_type.green]
				)

				line = line .. get_signal_values_csv(
					used_signals[defines.wire_type.green],
					circuit_logger.binary_signals
				)
			end
			if (circuit_logger.log_combined) then
				used_signals[defines.wire_type.combined] = combine_signals(
					used_signals[defines.wire_type.red],
					used_signals[defines.wire_type.green]
				)

				line = line .. get_signal_values_csv(
					used_signals[defines.wire_type.combined],
					circuit_logger.binary_signals
				)
			end
		end
	end

	return line
end

-- dumps first pending log to a file
local function dump_pending()
	if (#global.cl_status.to_dump>0) then
		local log = global.cl_status.to_dump[1]
		local filename = log.start_tick .. "-" .. log.end_tick

		local pt_st = get_fmt_play_time(log.start_tick)
		local pt_end = get_fmt_play_time(log.end_tick)

		data = "Start tick: ".. log.start_tick .. "(" .. pt_st .. ")" .. EOL
		data = data .. "End tick: " .. log.end_tick .. "(" .. pt_end .. ")" .. EOL
		data = data .. "Trigger on: " .. log.trigger_on_name .. EOL
		data = data .. "Trigger off: " .. log.trigger_off_name .. EOL

		data = data .. get_dump_headers(log.start_tick) .. EOL

		for tick, _ in pairs(log.changes) do
			data = data .. get_dump_line(tick, log.start_tick) .. EOL
		end

	 	game.write_file("circuit_logger/" .. filename .. ".csv", data)

		table.remove(global.cl_status.to_dump, 1)
		for _, circuit_logger in ipairs(global.cls) do
			circuit_logger.current_dump_log = nil
			if (circuit_logger.to_dump[1] ~= nil and circuit_logger.to_dump[1].start_tick==log.start_tick) then
				table.remove(circuit_logger.to_dump, 1)
			end
		end
	end
end

-- logs different signals into log table
local function log_diff_signals(diff_signals, circuit_logger, wire_type, tick)
	local data = circuit_logger.current_log.data
	data[tick] = data[tick] or {}
	data[tick][wire_type] = diff_signals
	global.cl_status.current_log.changes[tick] = true
	add_to_signal_list(diff_signals, circuit_logger.current_log.used_signals, wire_type)
	if (tick ~= global.cl_status.current_log.last_logged_tick) then
		global.cl_status.current_log.last_logged_tick = tick
		global.cl_status.current_log.count = global.cl_status.current_log.count + 1
	end
end

-- logs signals of one wire of logger, returns true if there is some change of signals
local function log_logger_wire(circuit_logger, wire_type, tick)
	local signals_new = get_wire_signals(circuit_logger.entity, wire_type)
	local signals_old = circuit_logger.last_signals[wire_type]

	if (signals_new == nil) then
		signals_new = {}
	end
	if (not compare_signals(signals_old, signals_new)) then
		local diff = changed_signals(signals_old, signals_new, circuit_logger.ignored_signals)
		if (diff ~= false) then
			log_diff_signals(diff, circuit_logger, wire_type, tick)
		end
		circuit_logger.last_signals[wire_type] = signals_new
	end
end

-- logs circuit signals of one logger
local function log_logger_signals(circuit_logger, tick)
	if (circuit_logger.log_green or circuit_logger.log_combined) then
		log_logger_wire(circuit_logger, defines.wire_type.green, tick)
	end
	if (circuit_logger.log_red or circuit_logger.log_combined) then
		log_logger_wire(circuit_logger, defines.wire_type.red, tick)
	end
end

-- logs circuit signals of all connected loggers
local function log_loggers_signals(event)
	if (global.cl_status.logging) then
		for _, circuit_logger in ipairs(global.cls) do
			log_logger_signals(circuit_logger, event.tick)
		end
	end
end

local function remove_invalid_loggers()
	for i=#global.cls,1,-1 do
		if not is_valid(global.cls[i].entity) then
			debugDump("Removed logger "..i, true)
			table.remove(global.cls, i)
		end
	end

	i = 0
	for _, v in pairs(global.cls) do
		i = i + 1
		v.column_number = i
	end
end

local function check_trigger_conditions(event)
	local is_trigger_off = false
	local is_trigger_on = false
	local logging_on = false
	local trigger_on_name = {}
	local trigger_off_name = {}

	--check loggers circuit contitions
	for i, circuit_logger in ipairs(global.cls) do
		if
			(circuit_logger.trigger_on or circuit_logger.trigger_off or circuit_logger.logging_on)
			and is_valid(circuit_logger.entity)
		then
			local cond_state = get_condition_state(circuit_logger.entity)
			if (cond_state and circuit_logger.logging_on) then
				logging_on = true
				if (not global.cl_status.logging) then
					is_trigger_on = true
					table.insert(trigger_on_name, circuit_logger.name or ("#"..i))
				end
			end
			if (cond_state and not circuit_logger.last_condition_state) then
				-- condition was just met
				if (circuit_logger.trigger_on) then
					is_trigger_on = true
					table.insert(trigger_on_name, circuit_logger.name or ("#"..i))
				end
				if (not logging_on and circuit_logger.trigger_off) then
					is_trigger_off = true
					table.insert(trigger_off_name, circuit_logger.name or ("#"..i))
				end
			elseif (not cond_state and circuit_logger.last_condition_state) then
			 -- condition is now not fulfilled
				if (circuit_logger.logging_on and not logging_on) then
					is_trigger_off = true
					table.insert(trigger_off_name, circuit_logger.name or ("#"..i))
				end
			end
			circuit_logger.last_condition_state = cond_state
		end
	end

	-- check global time based off trigger
	if
		is_trigger_off == false
		and global.cl_status.logging and global.cl_status.trigger_off_after
		and not is_trigger_on
		and not logging_on
		and global.cl_status.last_trigger_on+global.cl_status.trigger_off_ticks<=event.tick
	then
		is_trigger_off = true
		table.insert(trigger_off_name, "Time")
	end

	if (is_trigger_off and not logging_on and global.cl_status.logging) then
		trigger_off(event, join(trigger_off_name,","))
	end
	if (is_trigger_on and not global.cl_status.logging) then
		trigger_on(event, join(trigger_on_name,","))
	end
	if (is_trigger_on or is_logging) then
		global.cl_status.last_trigger_on = event.tick
	end
end

-- ---------------- EVENTS ---------------------

function on_tick(event)
	if global.cls == nil or #global.cls == 0 then return end

	if event.tick%10==7 then
		for _, player in pairs(game.players) do
			if is_valid(player.opened) and player.opened.name == "circuit-logger" then
				if not player.gui.left.circuit_logger then
					new_circuit_logger_gui(player, find_circuit_logger(global.cls, player.opened)[2])
				end
			elseif player.gui.left.circuit_logger ~= nil then
				player.gui.left.circuit_logger.destroy()
			end
		end
	end

	check_trigger_conditions(event)

	if global.cl_status.logging then
		log_loggers_signals(event)
	end

	if (event.tick%30==17) and (#global.cl_status.to_dump>0) then
		dump_pending()
	end

	if (event.tick%60==42) and global.cl_status.logging then
		update_gui_players(true)
	end

end

function on_gui_click(event)
	local player = game.players[event.player_index]
	local element = event.element
	local name = element.name

	local entity
	if is_valid(player.opened) and player.opened.name == "circuit-logger" then
		entity = player.opened
	end

	if entity ~= nil and name == "cl_save" then
		circuit_logger = find_circuit_logger(global.cls, player.opened)[2]
		update_circuit_logger(player.gui.left.circuit_logger, circuit_logger)
		circuit_logger.last_condition_state = false
	elseif (name == "circuit_logger_button") then
		expand_gui(player)
	elseif (name == "circuit_logger_trigger_on") then
		trigger_on(event, "Manually")
	elseif (name == "circuit_logger_trigger_off") then
		trigger_off(event, "Manually")
	elseif (name == "cl_save_settings") then
		update_settings(player)
	end

end

function on_init()
	initGlob()
	for _, player in pairs(game.players) do
		init_gui(player)
	end
end

function on_configuration_changed(event)
	initGlob()
	for _, player in pairs(game.players) do
		init_gui(player)
	end
end

function on_remove_entity(event)
	remove_invalid_loggers()
end

function on_built_entity(event)
	if is_valid(event.created_entity) and event.created_entity.name == "circuit-logger" then

		circuit_logger = {}
		circuit_logger.trigger_on = false
		circuit_logger.trigger_off = false
		circuit_logger.logging_on = false
		circuit_logger.name = ""

		circuit_logger.column_number = 1
		for _, __ in pairs(global.cls) do
			circuit_logger.column_number = circuit_logger.column_number + 1
		end

		circuit_logger.log_red = false
		circuit_logger.log_green = false
		circuit_logger.log_combined = false
		circuit_logger.disabled = false

		circuit_logger.entity = event.created_entity
		circuit_logger.position = event.created_entity.position

		circuit_logger.last_condition_state = false
		circuit_logger.condition_state = false
		circuit_logger.current_log = {}
		circuit_logger.to_dump = {}
		circuit_logger.ignored_signals = {}
		circuit_logger.signal_aliases = {}
		circuit_logger.binary_signals = {}

		player = circuit_logger.player
		circuit_logger.surface = circuit_logger.entity.surface
		surface = circuit_logger.surface

		table.insert(global.cls, circuit_logger)
		i = 0
		for _, v in pairs(global.cls) do
			i = i + 1
			v.column_number = i
		end
	end

end


script.on_event(defines.events.on_gui_click, on_gui_click)
script.on_event(defines.events.on_tick, on_tick)

script.on_init(on_init)
script.on_configuration_changed(on_configuration_changed)

script.on_event(defines.events.on_built_entity, on_built_entity)
script.on_event(defines.events.on_robot_built_entity, on_built_entity)

script.on_event(defines.events.on_entity_died, on_remove_entity)
script.on_event(defines.events.on_player_mined_item, on_remove_entity)
script.on_event(defines.events.on_robot_mined, on_remove_entity)

script.on_event(defines.events.on_research_finished, function(event)
	if (event.research.name == "circuit-logger") then
		for _, player in pairs(game.players) do
			if (event.research.force.name == player.force.name) then
				init_gui(player)
			end
		end
	end
end)

-- ----------- GUI --------------------

function get_status_text()
	local txtstatus
	if (global.cl_status.logging) then txtstatus = {"cl-status-logging"} else txtstatus = {"cl-status-stopped"} end
	return txtstatus
end

function get_trigger_text()
	if (global.cl_status.logging) then
		return global.cl_status.current_log.trigger_on_name
	else
		if (global.cl_status.last_log ~= nil) then
			return global.cl_status.last_log.trigger_off_name
		end
	end
	return ""
end

function get_count_records()
	if (global.cl_status.logging) then
		return global.cl_status.current_log.count
	else
		return ""
	end
end

function init_gui(player)
	if (not player.force.technologies["circuit-logger"].researched) then
		return
	end

	if (not player.gui.top["circuit_logger_button"]) then
		player.gui.top.add{
			type="button",
			name="circuit_logger_button",
			style="circuit_logger_main_menu_open_button_style",
			tooltip={"cl-dlg-settings"}
		}
	end
end

function expand_gui(player)
	local frame = player.gui.left.circuit_logger_settings
	if (frame) then
		frame.destroy()
	else
		frame = player.gui.left.add
		{
			type="frame",
			name="circuit_logger_settings",
			direction="vertical",
			caption={"cl-dlg-settings"}
		}

		global.cl_status_fl = frame.add{type="flow", name="status_fl", direction="horizontal"}
		global.cl_status_fl.add{type="label", name="cl_status_label", caption={"circuit-logger-status"}}
		global.cl_status_fl.add{type="label", name="cl_status", caption=get_status_text()}
		global.cl_status_fl.add{type="label", caption="("}
		global.cl_status_fl.add{type="label", name="circuit_logger_trigger_name", caption=get_trigger_text()}
		global.cl_status_fl.add{type="label", caption="), "}
		global.cl_status_fl.add{type="label", caption={"cl-logged-records"}}
		global.cl_status_fl.add{type="label", name="circuit_logger_records", caption=get_count_records()}

		trigger_fl = frame.add{type="flow", name="trigger_fl", direction="horizontal"}
		trigger_fl.add
		{
			type="checkbox",
			name="cl_trigger_off_after",
			caption={"cl-trigger-off-after-ticks"},
			state = global.cl_status.trigger_off_after==true
		}
		trigger_fl.add{type="textfield", name="cl_trigger_off_ticks", text = global.cl_status.trigger_off_ticks}

		ignored_fl = frame.add{type="flow", name="ignored_fl", direction="horizontal"}
		ignored_fl.add{type="label", caption={"cl-ignored-signals-caption"}}
		ignored_fl.add{type="textfield", name="cl_ignored_main", text=join(global.cl_status.ignored_signals,",")}



		buttons = frame.add{type="flow", name="buttons", direction="horizontal"}
		buttons.add{type="button", name="cl_save_settings", caption={"cl-button-save"}}
		buttons.add{type="button", name="circuit_logger_trigger_on", caption={"cl-checkbox-trigger-on"}}
		buttons.add{type="button", name="circuit_logger_trigger_off", caption={"cl-checkbox-trigger-off"}}
	end
end

function update_settings(player)
	local frame = player.gui.left.circuit_logger_settings
	if (frame) then
		global.cl_status.ignored_signals = split_string(frame.ignored_fl.cl_ignored_main.text, ",")
		global.cl_status.trigger_off_ticks = tonumber(frame.trigger_fl.cl_trigger_off_ticks.text)
		if global.cl_status.trigger_off_ticks~=nil then
			if global.cl_status.trigger_off_ticks<1 then global.cl_status.trigger_off_ticks = 1 end
			global.cl_status.trigger_off_after = frame.trigger_fl.cl_trigger_off_after.state
		else
			global.cl_status.trigger_off_after = false
			global.cl_status.trigger_off_ticks = ""
		end

		update_gui_players()
	end
end

function update_gui_players(only_info)
	for _, player in pairs(game.players) do
		update_gui(player, only_info)
	end
end

function update_gui(player, only_info)
	local frame = player.gui.left.circuit_logger_settings
	if (frame) then
		frame.status_fl.cl_status.caption = get_status_text()
		frame.status_fl.circuit_logger_trigger_name.caption = get_trigger_text()
		frame.status_fl.circuit_logger_records.caption = get_count_records()
		if (not only_info) then
			frame.trigger_fl.cl_trigger_off_after.state = global.cl_status.trigger_off_after
			frame.trigger_fl.cl_trigger_off_ticks.text = global.cl_status.trigger_off_ticks or ""
		end
	end
end

function new_circuit_logger_gui(player, circuit_logger)
	if (is_valid(player) and circuit_logger ~= nil) then
		player_gui = player.gui.left

		gui = gui_or_new(player_gui, {
			type="frame",
			name="circuit_logger",
			caption={"cl-window-title"},
			direction="vertical" }
		)

	 	name_flow = gui_or_new(gui, {type="flow", name="name_flow", direction="horizontal"})
		gui_or_new(name_flow, {type="label", name="cl_name_label", caption={"cl-name-caption"}})
		gui_or_new(name_flow, {type="textfield", name="cl_name", text=circuit_logger.name})

		gui_or_new(name_flow, {type="label", name="cl_column_number_label", caption={"cl-column-number-caption"}})
		gui_or_new(name_flow, {type="textfield", name="cl_column_number", text=circuit_logger.column_number})

		checkboxes = gui_or_new(gui, {type="flow", name="checkboxes", direction="horizontal"})

		gui_or_new(checkboxes, {
			type="checkbox",
			name="cl_trigger_on",
			caption={"cl-checkbox-trigger-on"},
			state = circuit_logger.trigger_on
		})
		gui_or_new(checkboxes, {
			type="checkbox",
			name="cl_trigger_off",
			caption={"cl-checkbox-trigger-off"},
			state = circuit_logger.trigger_off
		})
		gui_or_new(checkboxes, {
			type="checkbox",
			name="cl_logging_on",
			caption={"cl-checkbox-logging-on"},
			state = circuit_logger.logging_on
		})

		checkboxes2 = gui_or_new(gui, {
			type="flow",
			name="checkboxes2",
			direction="horizontal"
		})
		gui_or_new(checkboxes2, {
			type="checkbox",
			name="cl_log_red",
			caption={"cl-checkbox-log-red"},
			state = circuit_logger.log_red
		})
		gui_or_new(checkboxes2, {
			type="checkbox",
			name="cl_log_green",
			caption={"cl-checkbox-log-green"},
			state = circuit_logger.log_green
		})
		gui_or_new(checkboxes2, {
			type="checkbox",
			name="cl_log_combined",
			caption={"cl-checkbox-log-combined"},
			state = circuit_logger.log_combined
		})

	 	ignored_flow = gui_or_new(gui, {
			type="flow",
			name="ignored_flow",
			direction="horizontal"
		})
		gui_or_new(ignored_flow, {
			type="label",
			name="cl_ignored_label",
			caption={"cl-ignored-signals-caption"}
		})
		gui_or_new(ignored_flow, {
			type="textfield",
			name="cl_ignored",
			text=join(circuit_logger.ignored_signals,",")
		})

	 	aliases_flow = gui_or_new(gui, {
			type="flow",
			name="aliases_flow",
			direction="horizontal"
		})
		gui_or_new(aliases_flow, {
			type="label",
			name="cl_aliases_label",
			caption={"cl-signal-aliases-caption"}
		})
		gui_or_new(aliases_flow, {
			type="textfield",
			name="cl_aliases",
			text=join_key_value(circuit_logger.signal_aliases,"=",",")
		})

		buttons = gui_or_new(gui, {type="flow", name="buttons", direction="horizontal"})
		gui_or_new(buttons, {type="button", name="cl_save", caption={"cl-button-save"}, })

	end
	return gui
end

function parse_binary_signals_settings(circuit_logger)
	circuit_logger.binary_signals = {}
	for name, value in pairs(circuit_logger.signal_aliases) do
		if (string.find(value,"{") == 1) then
			local text = string.sub(value, 2, string.len(value)-1)
			circuit_logger.binary_signals[name] = split_key_value_string(text,";",":")
		end
	end
end

function update_circuit_logger(gui, circuit_logger)
	if gui ~= nil and circuit_logger ~= nil then
		circuit_logger.trigger_on = gui.checkboxes.cl_trigger_on.state
		circuit_logger.trigger_off = gui.checkboxes.cl_trigger_off.state
		circuit_logger.logging_on = gui.checkboxes.cl_logging_on.state
		circuit_logger.log_green = gui.checkboxes2.cl_log_green.state
		circuit_logger.log_red = gui.checkboxes2.cl_log_red.state
		circuit_logger.log_combined = gui.checkboxes2.cl_log_combined.state
		circuit_logger.name = gui.name_flow.cl_name.text

		local text = gui.ignored_flow.cl_ignored.text
		circuit_logger.ignored_signals = split_string(text, ",")
		text = gui.aliases_flow.cl_aliases.text
		circuit_logger.signal_aliases = split_key_value_string(text,",","=")

		parse_binary_signals_settings(circuit_logger)
		circuit_logger.last_condition_state = get_condition_state(circuit_logger.entity)

		if
			circuit_logger.column_number ~= gui.name_flow.cl_column_number.text
			and tonumber(gui.name_flow.cl_column_number.text) ~= nil
		then
			i = 0
			for _, __ in pairs(global.cls) do
				i = i + 1
			end

			if tonumber(gui.name_flow.cl_column_number.text) > i then
				gui.name_flow.cl_column_number.text = tostring(circuit_logger.column_number)
			else
				num1 = tonumber(circuit_logger.column_number)
				num2 = tonumber(gui.name_flow.cl_column_number.text)

				global.cls[num1], global.cls[num2] = global.cls[num2], global.cls[num1]

				global.cls[num1].column_number = num1
				global.cls[num2].column_number = num2
			end
		else
			gui.name_flow.cl_column_number.text = tostring(circuit_logger.column_number)
		end
	end
end

